/** Class representing a deck of cards
 *  In this case it represents the standard deck
 *  of 52 cards.  The cards are stored in the LList 
 *  class.
 */
public class Deck{

    private LList<Card> cards;

    public static char[] SUITS = {'C', 'D', 'H', 'S'};
    public static final int MIN_RANK = 1;
    public static final int MAX_RANK = 13;

    /** Constructs a standard 52 card deck with the
     *  cards in a random order.
     */
    public Deck(){
	cards = new LList();
	LList<Card> tmp = new LList(); // tmp list for the 
	                               // original ordered deck

	for (int r = MIN_RANK; r <= MAX_RANK; r++){
	    for (int s = 0; s < SUITS.length; s++){
		tmp.insertFront(new Card(r,SUITS[s]));
	    }
	} 

	java.util.Random randgen = new java.util.Random();

	// remove the cards at random from tmp and 
	// put them at the beginning of cards.
	for (int remaining_cards = tmp.size(); 
	     remaining_cards > 0; 
	     remaining_cards--){
	    int index = randgen.nextInt(remaining_cards);
	    Card c = tmp.removeIth(index);
	    cards.insertFront(c);
	}
    }

    /** @return the card on the top of the deck 
     */
    public Card draw(){
	if (cards.isEmpty()) return null;
	return cards.removeFront();
    }

    /** @return the number of cards remaining in the deck.
     */
    public int numCards(){
	return cards.size();
    }

    /** @return true iff the deck has no cards remaining.
     */
    public boolean isEmpty(){
	return cards.isEmpty();
    }

}
