/** Main class for this program.  Uses the Player class
 *  to track player information and the Deck class to 
 *  maintain the pile of shared cards.
 *  @author mwa
 */
public class GoFish{

    public static java.util.Scanner stdin = new java.util.Scanner(System.in);

    public static int CARDS_PER_PLAYER = 7;

    /** Asks a human player for a valid rank in his hand.
     *  Prompts user again if input is invalid.
     *  @param the player's object
     *  @return a rank from the player's hand
     */
    public static int getValidRank(Player p){
	
	while(true){
	    System.out.println("Please enter a valid rank");
	    String line = stdin.nextLine();
	    try{
		int r = Integer.parseInt(line);
		// it was an int
		if (p.hasRank(r)) 
		    // it was an int and a rank from their hand
		    return r;
	    } catch (Exception e){ }
	}
    }

    /** The main method.  Contains game setup and main loop
     */
    public static void main(String[] args){
	
	// Setup the players
	Player human = new Player();
	Player comp = new Player();

	// Setup the deck
	Deck d = new Deck();

	// Deal the cards
	System.out.println("Welcome to Go Fish");
	System.out.println("Dealing the cards");
	for (int i = 0; i < CARDS_PER_PLAYER; i++){
	    Card c = human.takeCard(d.draw());
	    if (c != null) // check whether a pair is created
		System.out.println("\tYou place a pair of " + c.getRank() + "'s");
	    c = comp.takeCard(d.draw());
	    if (c != null) // check whether a pair is created
		System.out.println("\tComputer places a pair of " + c.getRank() + "'s");
	}

	boolean human_turn = true;

	while (human.hasCards() && comp.hasCards()){
	    // loop while both players have cards
	    if (human_turn){
		// human's turn
		while(human.hasCards()){
		    // display game information
		    System.out.println("Your turn:");
		    System.out.println("\tYour Hand:");
		    System.out.println("\t\t" + human.getHand());
		    System.out.println("\tYour pairs: " + human.getPairs());
		    System.out.println("\tComputer pairs: " + comp.getPairs() 
				       + "\t Computer Cards: " 
				       + comp.getNumCards());
		    int rank = getValidRank(human); // get player move
		    System.out.println("\tYou say \"Do you have a " 
				       + rank + "?\"");
		    Card c2 = human.askForCards(comp,rank); 
		    if (c2 == null){
			// Go Fish
			System.out.println("\tComputer says \"Go Fish\"");
			if (!d.isEmpty()){
			    Card c = d.draw();
			    System.out.println("\tYou draw "+ c + ". " 
					       + d.numCards() 
					       + " card(s) remain.");
			    c = human.takeCard(c);
			    if (c == null)
				break;
			    else
				System.out.println("\tYou place a pair of " 
						   + c.getRank() + "'s");
			} else 
			    break;
		    } else {
			// Player gets card from computer
			System.out.println("\tComputer gives you " + c2);
			System.out.println("\tYou place a pair of " 
					   + c2.getRank() + "'s");
		    }

		}
	    } else {
		// Computer's turn
		while(comp.hasCards()){
		    System.out.println("Computer's turn:");
		    int rank = -1;
		    java.util.Random randgen = new java.util.Random();
		    while (!comp.hasRank(rank)){
			rank = randgen.nextInt(14);
		    }
		    System.out.println("\tComputer says \"Do you have a " 
				       + rank+"?\"");
		    Card c2 = comp.askForCards(human,rank);
		    if (c2 == null){
			// Go Fish
			System.out.println("\tYou say \"Go Fish\"");
			if (!d.isEmpty()){
			    System.out.println("\tComputer draws a card. " 
					       + (d.numCards()-1) 
					       + " card(s) remain.");
			    Card c = d.draw();
			    c = comp.takeCard(c);
			    if (c == null)
				break;
			    else
				System.out.println("\tComputer places a pair of " 
						   + c.getRank() + "'s");
			} else
			    break;
		    } else {
			// Player gives computer a card
			System.out.println("\tYou give the computer " + c2);
			System.out.println("\tComputer places a pair of " 
					   + c2.getRank() +"'s");
		    }
		}
	    }
	    human_turn = !human_turn; // switch turns
	}

	// Output winner
	System.out.println("Final Score: " + human.getPairs() 
			   + " to " + comp.getPairs() + ".");

	if (human.getPairs() > comp.getPairs())
	    System.out.println("You win!");
	else if (human.getPairs() == comp.getPairs())
	    System.out.println("It's a tie!");
	else 
	    System.out.println("Computer wins!");

    }

}
