/** Generic linked list class, uses LLNode as
 *  a substructure.
 *  @author mwa
 */
public class LList<E>{

    private LLNode<E> head = null;
    private int els = 0;

    /** @return the number of elements in the list
     */
    public int size(){
	return els;
    }

    /** @return true iff the list is empty
     */
    public boolean isEmpty(){
	return els == 0;
    }

    /** inserts in front of the first element in list
     *  @param item to add
     */
    public void insertFront(E el){
        head = new LLNode(el,head);
	els++;
    }

    /** removes and returns the first element of the list
     *  returns null if empty.
     *  @return item at front
     */
    public E removeFront(){
	E ret = null;
	if (head != null){
	    ret = head.getElement();
	    head = head.getNext();
	    els--;
	}
	return ret;
    }

    /** Checks whether an item is in the list, uses equals().
     *  @param item to look up
     *  @return true iff the item is in the list
     */
    public boolean contains(E el){
	for (LLNode curr = head; curr != null; curr = curr.getNext())
	    if (curr.getElement().equals(el))
		return true;
	return false;
    }

    /** removes and returns the first element equal to the parameter
     *  returns null if it is not found, uses equals() to compare.
     *  @param item to remove
     *  @return the item removed
     */
    public E remove(E el){
	LLNode prev = null;
	// loop over all elements
	for (LLNode<E> curr = head; curr != null; curr = curr.getNext()){	    
	    if (curr.getElement().equals(el)){
		// found it
		// remove it
		if (prev == null)
		    head = curr.getNext();
		else
		    prev.setNext(curr.getNext());
		curr.setNext(null);
		els--;
		return curr.getElement(); 
	    }
	    prev = curr;
	}
	return null;
    }
    
    /** removes and returns the ith element in the list
     *  returns null if not found
     *  @param element to remove
     *  @return the item removed
     */
    public E removeIth(int i){
	if (i == 0)
	    // its the first one
	    return removeFront();
	else {
	    // its in the middle
	    LLNode prev = null;
	    int count = 0;
	    // loop to it
	    for (LLNode<E> curr = head; curr != null; curr = curr.getNext()){
		if (count == i){
		    prev.setNext(curr.getNext());
		    els--;
		    return curr.getElement();
		}
		prev = curr;
		count++;
	    }
	}
	// past the end
	return null;
    }

    /** return the contents of a list by calling toString on each element
     *  and concatenating them.
     *  @return String representation
     */
    public String toString(){
	String ret = "";
	for (LLNode curr = head; curr != null; curr = curr.getNext())
	    ret += curr.getElement() + " ";
	return ret;
    }

    

}
