/** Class representing each player.  Contains a list of 
 *  the card in the player's hand using our LList class
 *  also keeps track of the number of pairs the player
 *  has accumulated.
 *  @author mwa
 */
public class Player{

    private LList<Card> hand;
    private int pairs;
    
    /** Constructs a player with an empty hand.
     */	
    public Player(){
	hand = new LList();
    }

    /** Takes a card from some source
     *  returns null if this player does not have a 
     *  matching card.  If there is a matching card it 
     *  is returned and the number of pairs is increased
     *  by one.
     *  @param the new card
     *  @return a matching card from this player's hand
     */
    public Card takeCard(Card c){
	Card ret = hand.remove(c);

	if (ret != null)
	    pairs++;  // found a match
	else 
	    hand.insertFront(c); // no match put in hand

	return ret;
    }

    /** Asks another player for a card of some rank
     *  returns null if the opposing player didn't have
     *  the rank of card.
     *  @param the opposing player
     *  @param the rank to ask for
     *  @param the card the opposing player gives.
     */
    public Card askForCards(Player opp, int rank){
	Card c = new Card(rank,'X'); // make up some card
	Card cr = opp.hand.remove(c); // get card from opp

	if (cr == null)  // not found
	    return null;
	pairs++;
	hand.remove(c);
	return cr;
    } 
    
    /** @param rank to lookup
     *  @return true iff the player has that rank in hand
     */
    public boolean hasRank(int r){
	return hand.contains(new Card(r,'X'));
    }

    /** @return true iff the hand is not empty
     */
    public boolean hasCards(){
	return !hand.isEmpty();
    }

    /** @return the number of pairs created so far
     */
    public int getPairs(){
	return pairs;
    }

    /** @return the number of cards in hand
     */
    public int getNumCards(){
	return hand.size();
    }

    /** @return String representation of hand
     */
    public String getHand(){
	return "" + hand;
    }
}
